﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Diagnostics;
using System.Xml.Linq;
using System.Windows.Browser;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using DeepView;

namespace LiveSearch_SimpleImageSearch
{
    public partial class Page : UserControl
    {
        #region Properties

        /// <summary>
        /// 어플리케이션 ID (여기에서 발급 : http://search.live.com/developers/createapp.aspx)
        /// </summary>
        public readonly string APP_ID = "960B0BFD7406978473171FF134C86BA4D22AF66B";

        /// <summary>
        /// 한 페이지 당 검색 결과물 개수
        /// </summary>
        public readonly int PER_PAGE = 28;

        /// <summary>
        /// 현재 페이지
        /// </summary>
        public int CurrentPage { get; set; }

        /// <summary>
        /// 마지막으로 검색한 키워드
        /// </summary>
        public string LastKeyword { get; set; }

        #endregion Properties

        #region Constructors

        /// <summary>
        /// 기본생성자
        /// </summary>
        public Page()
        {
            InitializeComponent();

            // 각종 프로퍼티 초기화
            InitProperties();

            // 이벤트 연결
            Loaded += new RoutedEventHandler(Page_Loaded);
        }


        #endregion Constructors

        #region Methods

        /// <summary>
        /// 프로퍼티들을 초기화합니다.
        /// </summary>
        private void InitProperties()
        {
            CurrentPage = 0;
            LastKeyword = "";
        }

        /// <summary>
        /// 이미지를 검색합니다.
        /// </summary>
        /// <param name="keyword"></param>
        private void SearchImage(string keyword)
        {
            if (string.IsNullOrEmpty(keyword) == true)
                return;

            // 검색을 시도하는 검색어가 마지막 검색한 검색어와 다르면
            if (LastKeyword.CompareTo(keyword) != 0) 
            {
                // 기존 검색 결과 삭제
                wpanImages.Children.Clear();

                // 첫 페이지부터 다시 검색
                CurrentPage = 0;
                LastKeyword = keyword;
            }
            else // 아니면 페이지 증가
            {
                CurrentPage++;
            }

            // 검색 버튼을 잠금 (중복 호출 방지)
            LockSearchButton();

            // 검색 Url 템플릿 (http://msdn.microsoft.com/en-us/library/dd251090.aspx 참고)
            string searchUrlTemplate = "http://api.search.live.net/xml.aspx?AppId={0}&Query={1}&Sources=Image&Version=2.0&Market=en-us&Adult=Moderate&Image.Count={2}&Image.Offset={3}";
            string searchUrl = string.Format(searchUrlTemplate, APP_ID, HttpUtility.UrlEncode(keyword), PER_PAGE, CurrentPage * PER_PAGE);

            // 비동기로 검색
            WebClient wc = new WebClient();
            wc.DownloadStringCompleted += new DownloadStringCompletedEventHandler(wc_DownloadStringCompleted);
            wc.DownloadStringAsync(new Uri(searchUrl, UriKind.Absolute));
        }

        /// <summary>
        /// XML을 처리합니다.
        /// </summary>
        /// <param name="xml"></param>
        private void XmlToImages(string xml)
        {
            XDocument xDoc = XDocument.Parse(xml);
            if (xDoc == null)
                return;

            XNamespace mms = "http://schemas.microsoft.com/LiveSearch/2008/04/XML/multimedia";

            // LINQ 쿼리
            var query = from thumb in xDoc.Descendants(mms + "ImageResult")
                        where thumb.Element(mms + "Thumbnail").Element(mms + "ContentType").Value == "image/jpeg" ||
                              thumb.Element(mms + "Thumbnail").Element(mms + "ContentType").Value == "image/png"
                        select new
                        {
                            Url = thumb.Element(mms + "MediaUrl").Value,
                            ThumnailUrl = thumb.Element(mms + "Thumbnail").Element(mms + "Url").Value
                        };

            // LINQ 쿼리를 순회하며 Image 생성
            foreach (var item in query)
            {
                BitmapImage bitmap = new BitmapImage { UriSource = new Uri(item.ThumnailUrl, UriKind.Absolute) };
                Image image = new Image
                {
                    Source = bitmap,
                    Width = 80,
                    Height = 80,
                    Stretch = Stretch.UniformToFill,
                    Cursor = Cursors.Hand,
                    Tag = item.Url
                };

                image.MouseLeftButtonUp += new MouseButtonEventHandler(image_MouseLeftButtonUp);
                wpanImages.Children.Insert(0, image); // 맨 앞에 삽입
            }
        }

        void image_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            string imageUrl = (sender as Image).Tag.ToString();
            if (string.IsNullOrEmpty(imageUrl) == true)
                return;

            HtmlPage.Window.Navigate(new Uri(imageUrl, UriKind.Absolute), "_blank");
        }

        /// <summary>
        /// 검색 버튼을 잠급니다.
        /// </summary>
        private void LockSearchButton()
        {
            buttonSearch.IsEnabled = false;
        }

        /// <summary>
        /// 검색 버튼을 잠금해제합니다.
        /// </summary>
        private void UnlockSearchButton()
        {
            buttonSearch.IsEnabled = true;
        }

        #endregion Methods

        #region EventHandlers

        void Page_Loaded(object sender, RoutedEventArgs e)
        {
            // 블렌드에서는 작동이 안되게 하여 문제 생기지 않도록 함
            if (HtmlPage.IsEnabled == true)
            {
                // MouseWheelHelper를 생성하기 위한 타이머
                DispatcherTimer timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(500) };
                timer.Tick += new EventHandler(timer_Tick);
                timer.Start();
            }
        }

        void timer_Tick(object sender, EventArgs e)
        {
            (sender as DispatcherTimer).Stop();

            MouseWheelHelper wheel = new MouseWheelHelper(svImages);
            wheel.Moved += new EventHandler<MouseWheelEventArgs>(wheel_Moved);
        }

        void wheel_Moved(object sender, MouseWheelEventArgs e)
        {
            if (e.Delta == 0)
                return;

            // 미래의 스크롤을 계산
            double futureScroll = svImages.VerticalOffset - e.Delta * 30;

            // 트리밍 (스크롤 범위를 벗어난 값이 설정되지 않도록)
            futureScroll = Math.Min(Math.Max(futureScroll, 0), svImages.ScrollableHeight);

            // 스크롤
            svImages.ScrollToVerticalOffset(futureScroll);
        }

        private void buttonSearch_Click(object sender, RoutedEventArgs e)
        {
            SearchImage(tboxKeyword.Text);
        }

        void wc_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            UnlockSearchButton();

            if (e.Cancelled == true)
                return;

            if (e.Error != null)
            {
                Debug.WriteLine(e.Error.Message);
                return;
            }

            XmlToImages(e.Result);
        }

        private void tboxKeyword_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                SearchImage(tboxKeyword.Text);
            }
        }

        #endregion EventHandlers
    }
}
